<?php
/**
 * @package shippingMethod
 * @copyright Copyright 2003-2010 Zen Cart Development Team
 * @copyright Portions Copyright 2003 osCommerce
 * @license http://www.zen-cart.com/license/2_0.txt GNU Public License V2.0
 * @version $Id: perweightunit.php 15616 2010-03-06 04:07:11Z ajeh $
 */

/**
 * "Per Weight Unit" shipping module, allowing you to offer per-unit-rate shipping options
 */
class inxpress extends base
{
    /**
     * $code determines the internal 'code' name used to designate "this" payment module
     *
     * @var string
     */
    public $code;
    /**
     * $title is the displayed name for this payment method
     *
     * @var string
     */
    public $title;
    /**
     * $description is a soft name for this payment method
     *
     * @var string
     */
    public $description;
    /**
     * module's icon
     *
     * @var string
     */
    public $icon;
    /**
     * $enabled determines whether this module shows or not... during checkout.
     *
     * @var boolean
     */
    public $enabled;

    /**
     * Constructor
     *
     * @return perweightunit
     */
    function __construct()
    {
        global $order, $db;

        $this->code = 'inxpress';
        $this->title = INXPRESS_TEXT_TITLE;
        $this->description = INXPRESS_TEXT_DESCRIPTION;
        $this->sort_order = INXPRESS_SHIPPING_SORT_ORDER;
        $this->icon = '';
        $this->tax_class = INXPRESS_SHIPPING_TAX_CLASS;
        $this->tax_basis = 'Shipping';

        // disable only when entire cart is free shipping
        if (zen_get_shipping_enabled($this->code)) {
            $this->enabled = true;
        }

        if (($this->enabled == true) && ((int) INXPRESS_SHIPPING_ZONE > 0)) {
            $check_flag = false;
            $check = $db->Execute(
                "select zone_id from " . TABLE_ZONES_TO_GEO_ZONES . "
                             where geo_zone_id = '" . INXPRESS_SHIPPING_ZONE . "'
                             and zone_country_id = '" . $order->delivery['country']['id'] . "'
                             order by zone_id"
            );
            while (!$check->EOF) {
                if ($check->fields['zone_id'] < 1) {
                    $check_flag = true;
                    break;
                } elseif ($check->fields['zone_id'] == $order->delivery['zone_id']) {
                    $check_flag = true;
                    break;
                }
                $check->MoveNext();
            }

            if ($check_flag == false) {
                $this->enabled = false;
            }
        }
    }

    /**
     * Obtain quote from shipping system/calculations
     *
     * @param  string $method
     *
     * @return array
     */
    public function quote($method = '')
    {
        global $order, $shipping_weight, $db;
        $rates = array();
        $shippingRates = $this->collectRates($order, $db);
        if (!$shippingRates) {
            return false;
        }
        $title = $db->Execute("select configuration_value from " . TABLE_CONFIGURATION . " where configuration_key = 'INXPRESS_TITLE'");
        $title = $title->fields['configuration_value'];
        foreach ($shippingRates as $rate) {
            $rate = array(
                'id' => $rate['carrier'],
                'title' => $rate['service'] . ' (Transit Days: ' . $rate['days'] . ')',
                'cost' => $rate['price']
            );
            array_push($rates, $rate);
        }

        $this->quotes = array(
            'id' => $this->code,
            'module' => $title,
            'methods' => $rates
        );

        if ($this->tax_class > 0) {
            $this->quotes['tax'] = zen_get_tax_rate($this->tax_class, $order->delivery['country']['id'], $order->delivery['zone_id']);
        }

        if (zen_not_null($this->icon)) {
            $this->quotes['icon'] = zen_image($this->icon, $this->title);
        }
        return $this->quotes;
    }

    public function collectRates($order, $db)
    {
        $account = $db->Execute("select configuration_value from " . TABLE_CONFIGURATION . " where configuration_key = 'INXPRESS_ACCOUNT_NUMBER'");
        $account = $account->fields['configuration_value'];
        $country_row = $db->Execute("select countries_iso_code_2 from countries where countries_id=" . SHIPPING_ORIGIN_COUNTRY);
        $origin_country = $country_row->fields['countries_iso_code_2'];
        $weight = 0;
        $shippingPrice = array('price' => 0, 'days' => '0');

        $origin = array(
            'name'        => '',
            'address1'    => '',
            'address2'    => '',
            'city'        => '',
            'province'    => '',
            'phone'       => '',
            'country'     => $origin_country,
            'postal_code' => SHIPPING_ORIGIN_ZIP
        );

        $destination = array(
            'name'        => '',
            'address1'    => $order->delivery['street_address'],
            'address2'    => $order->delivery[''],
            'city'        => $order->delivery['city'],
            'province'    => $order->delivery['state_code'],
            'phone'       => '',
            'country'     => $order->delivery['country']['iso_code_2'],
            'postal_code' => $order->delivery['postcode']
        );

        $items = array();

        if ($order->products) {
            foreach ($order->products as $product) {
                $item = array();
                {

                    $dimweight = $db->Execute('select * from ' . INXPRESS_VARIENTS_TABLE . ' where product_id=' . (int) $product['id']);

                    if (!empty($dimweight)) {
                        $item['depth'] = $this->dim_converter($dimweight->fields['length']);
                        $item['width'] = $this->dim_converter($dimweight->fields['width']);
                        $item['height'] = $this->dim_converter($dimweight->fields['height']);
                    }
                    $item['weight'] = $this->calculate_weight_in_grams($product['weight']);
                    $item['quantity'] = $product['qty'];
                    $item['id'] = $product['id'];
                    $item['sku'] = '';
                    $item['name'] = $product['name'];

                    $code = 'P';

                    array_push($items, $item);
                }
            }

            $price = $this->calcRate($account, $origin, $destination, $items, $order);

            if ($price) {
                $shippingPrice = $price;
            } else {
                $shippingPrice = false;
            }
        } elseif (true) {
            if ($_SESSION['cart']->contents) {
                foreach ($_SESSION['cart']->contents as $id => $val) {
                    $product = $db->Execute(
                        "select pd.products_name, pd.products_description, pd.products_url,
                                      p.products_id, p.products_quantity, p.products_model,
                                      p.products_image, p.products_price, p.products_virtual, p.products_weight,
                                      p.products_date_added, p.products_last_modified,
                                      date_format(p.products_date_available, '%Y-%m-%d') as
                                      products_date_available, p.products_status, p.products_tax_class_id,
                                      p.manufacturers_id,
                                      p.products_quantity_order_min, p.products_quantity_order_units, p.products_priced_by_attribute,
                                      p.product_is_free, p.product_is_call, p.products_quantity_mixed,
                                      p.product_is_always_free_shipping, p.products_qty_box_status, p.products_quantity_order_max,
                                      p.products_sort_order,
                                      p.products_discount_type, p.products_discount_type_from,
                                      p.products_price_sorter, p.master_categories_id
                              from " . TABLE_PRODUCTS . " p, " . TABLE_PRODUCTS_DESCRIPTION . " pd
                              where p.products_id = '" . $id . "'
                              and p.products_id = pd.products_id
                              and pd.language_id = '" . (int) $_SESSION['languages_id'] . "'"
                    );
                    $item = array(
                        'id' => $id,
                        'quantity' => $val['qty'],
                        'weight' => $this->calculate_weight_in_grams($product->fields['products_weight']),
                        'sku' => '',
                        'name' => $product->fields['products_name']
                    );

                    {

                        $dimweight = $db->Execute('select * from ' . INXPRESS_VARIENTS_TABLE . ' where product_id=' . (int) $item['id']);

                        if (!empty($dimweight)) {
                            $item['depth'] = $this->dim_converter($dimweight->fields['length']);
                            $item['width'] = $this->dim_converter($dimweight->fields['width']);
                            $item['height'] = $this->dim_converter($dimweight->fields['height']);
                        }

                        $code = '';
                        if ($weight > 0.5) {
                            $code = 'P';
                        } elseif ($weight != 0 && $weight <= 0.5) {
                            $code = 'X';
                        }

                        array_push($items, $item);

                    }
                }

                $price = $this->calcRate($account, $origin, $destination, $items, $order);

                if ($price) {
                    $shippingPrice = $price;
                } else {
                    $shippingPrice = false;
                }
            }
        }

        return $shippingPrice;
    }

    public function calcRate($account, $origin, $destination, $items, $order)
    {
        global $db;
        $log = fopen("inxpress.log", "a");
        $check_query = $db->Execute("select configuration_value from " . TABLE_CONFIGURATION . " where configuration_key = 'INXPRESS_STORE_ID'");
        $store_id = $check_query->fields['configuration_value'];

        $dhl_query = $db->Execute("select configuration_value from " . TABLE_CONFIGURATION . " where configuration_key = 'INXPRESS_DHL_ACTIVE'");
        $dhl_active = $dhl_query->fields['configuration_value'];

        $ups_query = $db->Execute("select configuration_value from " . TABLE_CONFIGURATION . " where configuration_key = 'INXPRESS_UPS_ACTIVE'");
        $ups_active = $ups_query->fields['configuration_value'];

        $url = 'https://api.inxpressapps.com/carrier/v1/stores/' . $store_id . '/rates';

        $gateway = $origin['country'];

        $services = array();

        if ($dhl_active == "True")(
            array_push(
                $services,
                array(
                    'carrier' => 'DHL',
                    'service' => 'DHL Express'
                )
            )
        );

        if ($ups_active == "True")(
            array_push(
                $services,
                array(
                    'carrier' => 'UPS',
                    'service' => 'DHL Express'
                )
            )
        );

        $payload = json_encode(
            array(
            'account'     => $account,
            'gateway'     => $gateway,
            'services'    => $services,
            'origin'      => $origin,
            'destination' => $destination,
            'items'       => $items,
            )
        );

        $ch = curl_init($url);
        # Setup request to send json via POST.
        curl_setopt($ch, CURLOPT_POSTFIELDS, $payload);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type:application/json'));
        # Return response instead of printing.
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        # Send request.
        $data = curl_exec($ch);
        curl_close($ch);

        $responseArray = json_decode($data, true);
        $rates = array();

        if ( isset($responseArray['rates']) ) {
            foreach ($responseArray['rates'] as $response) {
                $rate = array();
                $handling = $this->getHandlingFee($order);
                $rate['price'] = ($response['total_price'] / 100) + $handling;
                $rate['days'] = $response['display_sub_text'];
                $rate['service'] = $response['display_text'];
                $rate['carrier'] = $response['carrier'];
                array_push($rates, $rate);
            }

            return $rates;
        } else {
            return false;
        }
    }

    public function calculate_weight_in_grams( $weight_in_uom )
    {
        $weight_unit   = TEXT_PRODUCT_WEIGHT_UNIT;

        switch ( $weight_unit ) {
            case 'oz':
                $weight = $weight_in_uom * 28.3495;
                break;
            case 'lbs':
                $weight = $weight_in_uom * 453.5920;
                break;
            case 'kgs':
                $weight = $weight_in_uom * 1000;
                break;
            default:
                $weight = $weight_in_uom;
        }

        return round($weight, 1);
    }

    public function dim_converter( $dim )
    {
        $dim_unit = 'in';

        switch ( $dim_unit ) {
            case 'in':
                $con_dim = $dim * 25.4;
                break;
            case 'cm':
                $con_dim = $dim * 10;
                break;
            case 'm':
                $con_dim = $dim * 1000;
                break;
            case 'yd':
                $con_dim = $dim * 914.4;
                break;
            default:
                $con_dim = $dim;
        }

        return round($con_dim, 1);
    }

    public function getHandlingFee($order)
    {
        global $db;
        $fee = 0;
        $hadlingfee = $db->Execute("select configuration_value from " . TABLE_CONFIGURATION . " where configuration_key = 'INXPRESS_HANDLING_FEE'");
        $hadlingfee = $hadlingfee->fields['configuration_value'];
        if (!$hadlingfee) {
            return 0;
        }
        $type = $db->Execute("select configuration_value from " . TABLE_CONFIGURATION . " where configuration_key = 'INXPRESS_HANDLING_TYPE'");
        $type = $type->fields['configuration_value'];
        if ($type == 'Fixed') {
            return $hadlingfee;
        } else {
            return ($order->info['subtotal'] * $hadlingfee) / 100;
        }

        return $fee;
    }

    /**
     * Check to see whether module is installed
     *
     * @return boolean
     */
    public function check()
    {
        global $db;

        if (!isset($this->_check)) {
            $check_query = $db->Execute("select configuration_value from " . TABLE_CONFIGURATION . " where configuration_key = 'INXPRESS_ACTIVE'");

            $this->_check = $check_query->RecordCount();
        }

        return $this->_check;
    }

    /**
     * Install the shipping module and its configuration settings
     */
    public function install()
    {
        global $db;
        $userList = zen_get_users();
        $firstname = $userList[0]['name'];
        $lastname = $userList[0]['name'];
        $countryName = $db->Execute(
            "select configuration_id, configuration_title, configuration_value, configuration_key,
                                        use_function from " . TABLE_CONFIGURATION . "
                                        where configuration_group_id = '1' and configuration_key='STORE_COUNTRY'
                                        "
        );
        $countryName = $this->getConfigurationValue($countryName);
        $region_name = $db->Execute(
            "select configuration_id, configuration_title, configuration_value, configuration_key,
                                        use_function from " . TABLE_CONFIGURATION . "
                                        where configuration_group_id = '1' and configuration_key='STORE_ZONE'
                                        "
        );
        $region_name = $this->getConfigurationValue($region_name);
        $city = $db->Execute(
            "select configuration_id, configuration_title, configuration_value, configuration_key,
                                        use_function from " . TABLE_CONFIGURATION . "
                                        where configuration_group_id = '1' and configuration_key='STORE_ZONE'
                                        "
        );
        $city = $this->getConfigurationValue($city);
        $zip = zen_get_configuration_key_value('SHIPPING_ORIGIN_ZIP');

        $phone = '';

        $email = $userList[0]['email'];

        $address = $db->Execute(
            "select configuration_id, configuration_title, configuration_value, configuration_key,
                                        use_function from " . TABLE_CONFIGURATION . "
                                        where configuration_group_id = '1' and configuration_key='STORE_NAME_ADDRESS'
                                        "
        );

        $address = $this->getConfigurationValue($address);

        $website = HTTP_SERVER . DIR_WS_CATALOG;
        $company = 'Zencart Store';

        $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Enabled', 'INXPRESS_ACTIVE','True', '', '6', '0', 'zen_cfg_select_option(array(\'True\', \'False\'), ', now())");
        $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('InXpress Account Number', 'INXPRESS_ACCOUNT_NUMBER', '', '', '6', '1', now())");
        $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('UPS Enabled', 'INXPRESS_UPS_ACTIVE','True', '', '6', '2', 'zen_cfg_select_option(array(\'True\', \'False\'), ', now())");
        $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('DHL Enabled', 'INXPRESS_DHL_ACTIVE','True', '', '6', '3', 'zen_cfg_select_option(array(\'True\', \'False\'), ', now())");
        $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Title', 'INXPRESS_TITLE', 'Rates by InXpress', '', '6', '4', now())");
        $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Handling Type', 'INXPRESS_HANDLING_TYPE','Fixed', '', '6', '5', 'zen_cfg_select_option(array(\'Fixed\', \'Percent\'), ', now())");
        $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Handling Fee', 'INXPRESS_HANDLING_FEE','', '', '6', '6', now())");
        $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, use_function, set_function, date_added) values ('Shipping Zone', 'INXPRESS_SHIPPING_ZONE', '0', 'If a zone is selected, only enable this shipping method for that zone.', '6', '7', 'zen_get_zone_class_title', 'zen_cfg_pull_down_zone_classes(', now())");
        $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Sort Order', 'INXPRESS_SHIPPING_SORT_ORDER', '0', 'Sort order of display.', '6', '8', now())");

        $check_query = $db->Execute("select configuration_value from " . TABLE_CONFIGURATION . " where configuration_key = 'INXPRESS_STORE_ID'");
        $store_id = $check_query->fields['configuration_value'];
        if (!$store_id) {
            $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('InXpress Store ID', 'INXPRESS_STORE_ID', '', 'Customer Portal store id', '6', '9', now())");
        }


        $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, use_function, set_function, date_added) values ('Tax Class', 'INXPRESS_SHIPPING_TAX_CLASS', '0', 'Use the following tax class on the shipping fee.', '6', '0', 'zen_get_tax_class_title', 'zen_cfg_pull_down_tax_classes(', now())");

        $db->Execute("insert into " . TABLE_ADMIN_MENUS . " (menu_key, language_key, sort_order) values ('inxpress', 'INXPRESS_MENU_TITLE', '1')");
        $varientTable = DB_PREFIX . 'inxpress_variant';
        $db->Execute(
            "
		CREATE TABLE IF NOT EXISTS `{$varientTable}` (
			`id` int(11) NOT NULL auto_increment,
			`product_id` int(11),
			`website` int(11),
			`store` int(11),
			`variant` text,
			`modifieddate` datetime,
			`length` text,
			`width` text,
			`height` text,
			`dim_weight` text,
			`variable` text,
			`extra` text,
			PRIMARY KEY  (`id`)
			) ENGINE=InnoDB DEFAULT CHARSET=utf8;
		"
        );
        if (function_exists('zen_register_admin_page')) {
            if (!zen_page_key_exists('inxpress_manage_varients')) {
                // Add Monthly Report link to Reports menu

                zen_register_admin_page('inxpress_manage_varients', 'INXPRESS_VARIENTS_TITLE', 'INXPRESS_FILE', 'set=varients', 'inxpress', 'Y', 17);
            }
            if (!zen_page_key_exists('inxpress_portal')) {
                // Add Monthly Report link to Reports menu

                zen_register_admin_page('inxpress_portal', 'INXPRESS_PORTAL_TITLE', 'INXPRESS_FILE', 'set=portal', 'inxpress', 'Y', 17);
            }
        }
    }

    public function getConfigurationValue($configuration)
    {
        if (zen_not_null($configuration->fields['use_function'])) {
            $use_function = $configuration->fields['use_function'];
            if (preg_match('/->/', $use_function)) {
                $class_method = explode('->', $use_function);
                if (!is_object(${$class_method[0]})) {
                    include DIR_WS_CLASSES . $class_method[0] . '.php';
                    ${$class_method[0]} = new $class_method[0]();
                }
                $cfgValue = zen_call_function($class_method[1], $configuration->fields['configuration_value'], ${$class_method[0]});
            } else {
                $cfgValue = zen_call_function($use_function, $configuration->fields['configuration_value']);
            }
        } else {
            $cfgValue = $configuration->fields['configuration_value'];
        }

        return $cfgValue;
    }

    /**
     * Remove the module and all its settings
     */
    public function remove()
    {
        global $db;
        $db->Execute("delete from " . TABLE_CONFIGURATION . " where configuration_key LIKE 'INXPRESS%' AND configuration_key !='INXPRESS_STORE_ID'");
        $db->Execute("delete from " . TABLE_ADMIN_PAGES . " where main_page = 'INXPRESS_FILE'");
        $db->Execute("delete from " . TABLE_ADMIN_MENUS . " where menu_key='inxpress'");
        $db->Execute("DROP TABLE  IF EXISTS " . INXPRESS_VARIENTS_TABLE);
    }

    /**
     * Internal list of configuration keys used for configuration of the module
     *
     * @return array
     */
    public function keys()
    {
        return array(
            'INXPRESS_ACTIVE',
            'INXPRESS_UPS_ACTIVE',
            'INXPRESS_DHL_ACTIVE',
            'INXPRESS_ACCOUNT_NUMBER',
            'INXPRESS_TITLE',
            'INXPRESS_HANDLING_TYPE',
            'INXPRESS_HANDLING_FEE',
            'INXPRESS_SHIPPING_ZONE',
            'INXPRESS_SHIPPING_SORT_ORDER',
            'INXPRESS_SHIPPING_TAX_CLASS',
        );
    }
}
